﻿using FlowerShopContracts.BindingModels;
using FlowerShopContracts.ViewModels;
using FlowerShopDataModels.Enums;
using FlowerShopDataModels;
using System.Xml.Linq;
using System.Runtime.Serialization;

namespace FlowerShopFileImplement.Models
{
	[DataContract]
	public class Order : IOrderModel
    {
		[DataMember]
		public int Id { get; private set; }
		[DataMember]
		public int FlowerId { get; private set; }
		[DataMember]
		public int? ImplementerId { get; private set; } = null;
		[DataMember]
		public OrderStatus Status { get; private set; }
		[DataMember]
		public int Count { get; private set; }
		[DataMember]
		public double Sum { get; private set; }
		[DataMember]
		public DateTime DateCreate { get; private set; }
		[DataMember]
		public DateTime? DateImplement { get; private set; }
		[DataMember]
		public int ClientId { get; private set; }

		public static Order? Create(OrderBindingModel model)
        {
            if (model == null)
            {
                return null;
            }
            return new Order()
            {
                Id = model.Id,
                FlowerId = model.FlowerId,
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement,
                ClientId = model.ClientId,
				ImplementerId = model.ImplementerId,

			};
        }
        public static Order? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }
            return new Order()
            {
                Id = Convert.ToInt32(element.Attribute("Id")!.Value),
                FlowerId = Convert.ToInt32(element.Element("FlowerId")!.Value),
				ImplementerId = Convert.ToInt32(element.Element("ImplementerId")!.Value),
				Count = Convert.ToInt32(element.Element("Count")!.Value),
                Sum = Convert.ToDouble(element.Element("Sum")!.Value),
                Status = (OrderStatus)Enum.Parse(typeof(OrderStatus), element.Element("Status")!.Value.ToString()),
                DateCreate = Convert.ToDateTime(element.Element("DateCreate")!.Value),
                ClientId = Convert.ToInt32(element.Element("ClientId")!.Value),
                DateImplement = string.IsNullOrEmpty(element.Element("DateImplement")!.Value) ? null : Convert.ToDateTime(element.Element("DateImplement")!.Value)
            };
        }
        public void Update(OrderBindingModel model)
        {
            if (model == null)
            {
                return;
            }
            Status = model.Status;
            DateImplement = model.DateImplement;
        }
        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            FlowerId = FlowerId,
            Count = Count,
            Sum = Sum,
            ClientId = ClientId,
			ImplementerId = ImplementerId,
			Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement,
        };
        public XElement GetXElement => new("Order",
        new XAttribute("Id", Id),
        new XElement("FlowerId", FlowerId),
        new XElement("Sum", Sum.ToString()),
        new XElement("Count", Count),
        new XElement("Status", Status.ToString()),
        new XElement("DateCreate", DateCreate.ToString()),
        new XElement("ClientId", ClientId),
		new XElement("ImplementerId", ImplementerId),
		new XElement("DateImplement", DateImplement.ToString())
        );

    }
}
