﻿using FlowerShopBusinessLogic;
using FlowerShopContracts.BindingModels;
using FlowerShopContracts.BusinessLogicsContracts;
using FlowerShopContracts.DI;
using FlowerShopDataModels;
using FlowerShopDataModels.Enums;
using Microsoft.Extensions.Logging;
using ProjectFlowerShop;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ProjectFlowerShop
{
	public partial class MainForm : Form
	{
		private readonly ILogger _logger;
		private readonly IOrderLogic _orderLogic;
		private readonly IReportLogic _reportLogic;
		private readonly IWorkProcess _workProcess;
		private readonly IBackUpLogic _backUpLogic;
		public MainForm(ILogger<MainForm> logger, IOrderLogic orderLogic, IReportLogic reportLogic,
			IWorkProcess workProcess, IBackUpLogic backUpLogic)
		{
			InitializeComponent();
			_logger = logger;
			_orderLogic = orderLogic;
			_reportLogic = reportLogic;
			_workProcess = workProcess;
			_backUpLogic = backUpLogic;
		}

		private void КомпонентыStripMenuItem_Click(object sender, EventArgs e)
		{
			var service = DependencyManager.Instance.Resolve<FormComponents>();
			if (service is FormComponents form)
			{
				form.ShowDialog();
			}

		}

		private void MainForm_Load(object sender, EventArgs e)
		{
			LoadData();
		}

		private void LoadData()
		{
			_logger.LogInformation("Загрузка заказов");
			try
			{
				DataGridView.FillandConfigGrid(_orderLogic.ReadList(null));
				_logger.LogInformation("Загрузка заказов");
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Ошибка загрузки заказов");
				MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void ЦветыStripMenuItem_Click(object sender, EventArgs e)
		{
			var service = DependencyManager.Instance.Resolve<FormFlowers>();
			if (service is FormFlowers form)
			{
				form.ShowDialog();
			}
		}

		private void CreateOrderButton_Click(object sender, EventArgs e)
		{
			var service = DependencyManager.Instance.Resolve<FormCreateOrder>();
			if (service is FormCreateOrder form)
			{
				form.ShowDialog();
				LoadData();
			}
		}

		private OrderBindingModel CreateBindingModel(int id, bool isDone = false)
		{
			return new OrderBindingModel
			{
				Id = id,
				FlowerId = Convert.ToInt32(DataGridView.SelectedRows[0].Cells["FlowerId"].Value),
				ClientId = Convert.ToInt32(DataGridView.SelectedRows[0].Cells["ClientId"].Value),
				Status = Enum.Parse<OrderStatus>(DataGridView.SelectedRows[0].Cells["Status"].Value.ToString()),
				Count = Convert.ToInt32(DataGridView.SelectedRows[0].Cells["Count"].Value),
				Sum = double.Parse(DataGridView.SelectedRows[0].Cells["Sum"].Value.ToString()),
				DateCreate = DateTime.Parse(DataGridView.SelectedRows[0].Cells["DateCreate"].Value.ToString()),
			};
		}

		private void TakeInWorkButton_Click(object sender, EventArgs e)
		{
			if (DataGridView.SelectedRows.Count == 1)
			{
				int id =
			   Convert.ToInt32(DataGridView.SelectedRows[0].Cells["Id"].Value);
				_logger.LogInformation("Заказ №{id}. Меняется статус на 'В работе'", id);
				try
				{
					var operationResult = _orderLogic.TakeOrderInWork(CreateBindingModel(id));
					if (!operationResult)
					{
						throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
					}
					LoadData();
				}
				catch (Exception ex)
				{
					_logger.LogError(ex, "Ошибка передачи заказа в работу");
					MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,
					MessageBoxIcon.Error);
				}
			}

		}

		private void ReadyButton_Click(object sender, EventArgs e)
		{
			if (DataGridView.SelectedRows.Count == 1)
			{
				int id =
			   Convert.ToInt32(DataGridView.SelectedRows[0].Cells["Id"].Value);
				_logger.LogInformation("Заказ №{id}. Меняется статус на 'Готов'",
			   id);
				try
				{
					var operationResult = _orderLogic.FinishOrder(CreateBindingModel(id));
					if (!operationResult)
					{
						throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
					}
					LoadData();
				}
				catch (Exception ex)
				{
					_logger.LogError(ex, "Ошибка отметки о готовности заказа");
					MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,
	MessageBoxIcon.Error);
				}
			}
		}

		private void IssuedButton_Click(object sender, EventArgs e)
		{
			if (DataGridView.SelectedRows.Count == 1)
			{
				int id =
			   Convert.ToInt32(DataGridView.SelectedRows[0].Cells["Id"].Value);
				_logger.LogInformation("Заказ №{id}. Меняется статус на 'Выдан'",
			   id);
				try
				{
					var operationResult = _orderLogic.DeliveryOrder(CreateBindingModel(id));
					if (!operationResult)
					{
						throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
					}
					_logger.LogInformation("Заказ №{id} выдан", id);
					LoadData();
				}
				catch (Exception ex)
				{
					_logger.LogError(ex, "Ошибка отметки о выдачи заказа");
					MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,
				   MessageBoxIcon.Error);
				}
			}

		}

		private void RefreshButton_Click(object sender, EventArgs e)
		{
			LoadData();
		}

		private void списокКомпонентовToolStripMenuItem_Click(object sender, EventArgs e)
		{
			using var dialog = new SaveFileDialog { Filter = "docx|*.docx" };
			if (dialog.ShowDialog() == DialogResult.OK)
			{
				_reportLogic.SaveComponentsToWordFile(new ReportBindingModel
				{
					FileName = dialog.FileName
				});
				MessageBox.Show("Выполнено", "Успех", MessageBoxButtons.OK,
			   MessageBoxIcon.Information);
			}
		}

		private void компонентыToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var service = DependencyManager.Instance.Resolve<FormReportFlowerComponent>();
			if (service is FormReportFlowerComponent form)
			{
				form.ShowDialog();
			}

		}

		private void списокЗаказовToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var service = DependencyManager.Instance.Resolve<FormReportOrders>();
			if (service is FormReportOrders form)
			{
				form.ShowDialog();
			}
		}

		private void клиентыToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var service = DependencyManager.Instance.Resolve<FormClients>();
			if (service is FormClients form)
			{
				form.ShowDialog();
			}
		}

		private void стартРаботToolStripMenuItem_Click(object sender, EventArgs e)
		{
			_workProcess.DoWork(DependencyManager.Instance.Resolve<IImplementerLogic>(), _orderLogic);
			MessageBox.Show("Процесс обработки запущен", "Сообщение",
		   MessageBoxButtons.OK, MessageBoxIcon.Information);
		}

		private void исполнителиToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var service = DependencyManager.Instance.Resolve<ImplementersForm>();
			if (service is ImplementersForm form)
			{
				form.ShowDialog();
			}
		}

		private void почтаToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var service = DependencyManager.Instance.Resolve<FormViewMail>();
			if (service is FormViewMail form)
			{
				form.ShowDialog();
			}
		}

		private void создатьБэкапToolStripMenuItem_Click(object sender, EventArgs e)
		{
			try
			{
				if (_backUpLogic != null)
				{
					var fbd = new FolderBrowserDialog();
					if (fbd.ShowDialog() == DialogResult.OK)
					{
						_backUpLogic.CreateBackUp(new BackUpSaveBinidngModel
						{
							FolderName = fbd.SelectedPath
						});
						MessageBox.Show("Бекап создан", "Сообщение",
					   MessageBoxButtons.OK, MessageBoxIcon.Information);
					}
				}
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,
			   MessageBoxIcon.Error);
			}
		}
	}
}
