﻿using FlowerShopBusinessLogic.BusinessLogic;
using FlowerShopBusinessLogic.MailWorker;

using FlowerShopContracts.BusinessLogicsContracts;
using FlowerShopContracts.StoragesContracts;
using FlowerShopDatabaseImplement.Implements;
using ProjectFlowerShop;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using NLog.Extensions.Logging;
using System;
using System.Drawing;
using FlowerShopBusinessLogic;
using FlowerShopBusinessLogic.OfficePackage.Implements;
using FlowerShopBusinessLogic.OfficePackage;
using FlowerShopContracts.BindingModels;
using FlowerShopContracts.DI;

namespace ProjectFlowerShop
{
    internal static class Program
    {

        /// <summary>
        ///  The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            ApplicationConfiguration.Initialize();
            var services = new ServiceCollection();
			InitDependency();
			try
			{
				var mailSender =
				DependencyManager.Instance.Resolve<AbstractMailWorker>();
				mailSender?.MailConfig(new MailConfigBindingModel
				{
					MailLogin =
				System.Configuration.ConfigurationManager.AppSettings["MailLogin"] ??
				string.Empty,
					MailPassword =
				System.Configuration.ConfigurationManager.AppSettings["MailPassword"] ??
				string.Empty,
					SmtpClientHost =
				System.Configuration.ConfigurationManager.AppSettings["SmtpClientHost"] ??
				string.Empty,
					SmtpClientPort =
				Convert.ToInt32(System.Configuration.ConfigurationManager.AppSettings["SmtpClientPort"]),
					PopHost =
				System.Configuration.ConfigurationManager.AppSettings["PopHost"] ?? string.Empty,
					PopPort =
				Convert.ToInt32(System.Configuration.ConfigurationManager.AppSettings["PopPort"])
				});

				var timer = new System.Threading.Timer(new
				TimerCallback(MailCheck!), null, 0, 100000);
			}
			catch (Exception ex)
			{
				var logger = DependencyManager.Instance.Resolve<ILogger>();
				logger?.LogError(ex, "Ошибка работы с почтой");
			}
			Application.Run(DependencyManager.Instance.Resolve<MainForm>());
		}

		private static void InitDependency()
		{
			DependencyManager.InitDependency();
			DependencyManager.Instance.AddLogging(option =>
			{
				option.SetMinimumLevel(LogLevel.Information);
				option.AddNLog("nlog.config");
			});
			DependencyManager.Instance.RegisterType<IClientLogic, ClientLogic>();
			DependencyManager.Instance.RegisterType<IImplementerLogic, ImplementerLogic>();
			DependencyManager.Instance.RegisterType<IComponentLogic, ComponentLogic>();
			DependencyManager.Instance.RegisterType<IOrderLogic, OrderLogic>();
			DependencyManager.Instance.RegisterType<IFlowerLogic, FlowerLogic>();
			DependencyManager.Instance.RegisterType<IReportLogic, ReportLogic>();
			DependencyManager.Instance.RegisterType<IWorkProcess, WorkProcess>();
			DependencyManager.Instance.RegisterType<IMessageInfoLogic, MessageInfoLogic>();
			DependencyManager.Instance.RegisterType<IBackUpLogic, BackUpLogic>();
			DependencyManager.Instance.RegisterType<AbstractMailWorker, MailKitWorker>();

			DependencyManager.Instance.RegisterType<AbstractSaveToExcel, SaveToExcel>();
			DependencyManager.Instance.RegisterType<AbstractSaveToWord, SaveToWord>();
			DependencyManager.Instance.RegisterType<AbstractSaveToPdf, SaveToPdf>();

			DependencyManager.Instance.RegisterType<MainForm>();
			DependencyManager.Instance.RegisterType<ImplementerForm>();
			DependencyManager.Instance.RegisterType<ImplementersForm>();
			DependencyManager.Instance.RegisterType<FormClients>();
			DependencyManager.Instance.RegisterType<ComponentForm>();
			DependencyManager.Instance.RegisterType<FormComponents>();
			DependencyManager.Instance.RegisterType<FormCreateOrder>();
			DependencyManager.Instance.RegisterType<FormFlower>();
			DependencyManager.Instance.RegisterType<FormFlowerComponent>();
			DependencyManager.Instance.RegisterType<FormFlowers>();
			DependencyManager.Instance.RegisterType<FormReportFlowerComponent>();
			DependencyManager.Instance.RegisterType<FormReportOrders>();
			DependencyManager.Instance.RegisterType<FormViewMail>();
		}

		private static void MailCheck(object obj) => DependencyManager.Instance.Resolve<AbstractMailWorker>()?.MailCheck();


	}
}
